#
# Copyright 2020 - Swiss Data Science Center (SDSC)
# A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
# Eidgenössische Technische Hochschule Zürich (ETHZ).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Renku service response json encoder."""
from datetime import datetime

from flask.json import JSONEncoder
from marshmallow.utils import isoformat


class SvcJSONEncoder(JSONEncoder):
    """Custom service json encoder."""

    def default(self, obj):
        """Overrides default json encoder with datetime iso format."""
        try:
            if isinstance(obj, datetime):
                return isoformat(obj)
            iterable = iter(obj)
        except TypeError:
            pass
        else:
            return list(iterable)
        return JSONEncoder.default(self, obj)



#
# Copyright 2020 - Swiss Data Science Center (SDSC)
# A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
# Eidgenössische Technische Hochschule Zürich (ETHZ).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Renku service datasets create controller."""
from renku.command.dataset import create_dataset_command
from renku.core.dataset.request_model import ImageRequestModel
from renku.core.util.metadata import construct_creators
from renku.ui.service.cache.models.job import Job
from renku.ui.service.config import CACHE_UPLOADS_PATH, MESSAGE_PREFIX
from renku.ui.service.controllers.api.abstract import ServiceCtrl
from renku.ui.service.controllers.api.mixins import RenkuOpSyncMixin
from renku.ui.service.controllers.utils.datasets import set_url_for_uploaded_images
from renku.ui.service.serializers.datasets import DatasetCreateRequest, DatasetCreateResponseRPC
from renku.ui.service.views import result_response


class DatasetsCreateCtrl(ServiceCtrl, RenkuOpSyncMixin):
    """Controller for datasets create endpoint."""

    REQUEST_SERIALIZER = DatasetCreateRequest()
    RESPONSE_SERIALIZER = DatasetCreateResponseRPC()

    def __init__(self, cache, user_data, request_data, migrate_project=False):
        """Construct a datasets create controller."""
        self.ctx = DatasetsCreateCtrl.REQUEST_SERIALIZER.load(request_data)
        self.ctx["commit_message"] = f"{MESSAGE_PREFIX} dataset create {self.ctx['name']}"

        super().__init__(cache, user_data, request_data, migrate_project)

    @property
    def context(self):
        """Controller operation context."""
        return self.ctx

    def renku_op(self):
        """Renku operation for the controller."""
        images = self.ctx.get("images")
        if images:
            user_cache_dir = CACHE_UPLOADS_PATH / self.user.user_id
            set_url_for_uploaded_images(images=images, cache=self.cache, user=self.user)

            images = [
                ImageRequestModel(
                    content_url=img["content_url"],
                    position=img["position"],
                    mirror_locally=img.get("mirror_locally", False),
                    safe_image_paths=[user_cache_dir],
                )
                for img in images
            ]

        creators = self.ctx.get("creators")
        if creators:
            creators, _ = construct_creators(creators)

        return (
            create_dataset_command()
            .with_commit_message(self.ctx["commit_message"])
            .build()
            .execute(
                name=self.ctx["name"],
                title=self.ctx.get("title"),
                creators=creators,
                description=self.ctx.get("description"),
                keywords=self.ctx.get("keywords"),
                images=images,
                custom_metadata=self.ctx.get("custom_metadata"),
                datadir=self.ctx.get("data_directory"),
            )
        )

    def to_response(self):
        """Execute controller flow and serialize to service response."""
        op_result, remote_branch = self.execute_and_sync()

        if isinstance(op_result, Job):
            return result_response(DatasetsCreateCtrl.JOB_RESPONSE_SERIALIZER, op_result)

        op_result = self.ctx
        op_result["remote_branch"] = remote_branch

        return result_response(DatasetsCreateCtrl.RESPONSE_SERIALIZER, op_result)



#  Copyright Swiss Data Science Center (SDSC). A partnership between
#  École Polytechnique Fédérale de Lausanne (EPFL) and
#  Eidgenössische Technische Hochschule Zürich (ETHZ).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Renku session commands."""

from renku.command.command_builder.command import Command
from renku.core.session.session import (
    search_session_providers,
    search_sessions,
    session_list,
    session_open,
    session_start,
    session_stop,
    ssh_setup,
)


def search_sessions_command():
    """Get all the session names that match a pattern."""
    return Command().command(search_sessions).require_migration().with_database(write=False)


def search_session_providers_command():
    """Get all the session provider names that match a pattern."""
    return Command().command(search_session_providers).require_migration().with_database(write=False)


def session_list_command():
    """List all the running interactive sessions."""
    return Command().command(session_list)


def session_start_command():
    """Start an interactive session."""
    return Command().command(session_start).with_database().require_migration()


def session_stop_command():
    """Stop a running an interactive session."""
    return Command().command(session_stop)


def session_open_command():
    """Open a running interactive session."""
    return Command().command(session_open)


def ssh_setup_command():
    """Setup SSH keys for SSH connections to sessions."""
    return Command().command(ssh_setup)



#
# Copyright 2018-2023- Swiss Data Science Center (SDSC)
# A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
# Eidgenössische Technische Hochschule Zürich (ETHZ).
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Base Renku persistent class."""

import persistent


class Persistent(persistent.Persistent):
    """Base Persistent class for renku classes.

    Subclasses are assumed to be immutable once persisted to the database. If a class shouldn't be immutable then
    subclass it directly from persistent.Persistent.
    """

    _v_immutable = False

    def reassign_oid(self):
        """Reassign ``oid`` (after assigning a new identifier for example)."""
        from renku.infrastructure.database import Database

        if self._p_jar is not None:
            self._p_jar.remove_from_cache(self)

        self._p_oid = None
        self._p_oid = Database.generate_oid(self)

    @property
    def immutable(self):
        """Return if object is immutable."""
        return self._v_immutable

    def freeze(self):
        """Set immutable property."""
        self._v_immutable = True

    def unfreeze(self):
        """Allows modifying an immutable object.

        Don't make an object mutable unless the intention is to drop the changes or modify the object in-place. Modified
        objects will be updated in-place which results in a binary diff when persisted. Normally, we want to create a
        mutable copy and persist it as a new object.
        """
        self._v_immutable = False

    def __setattr__(self, key, value):
        if self._v_immutable and key != "__weakref__" and not key.startswith("_p_") and not key.startswith("_v_"):
            raise RuntimeError(f"Cannot modify immutable object {self}.{key}")

        super().__setattr__(key, value)

    @property
    def __name__(self):
        return self.__class__.__name__
